@@ -2,14 +2,17 @@
 #this repository contains the full copyright notices and license terms.
 from decimal import Decimal
 import operator
-from trytond.model import ModelView, ModelSQL
-from trytond.pool import Pool
+from trytond.pool import Pool, PoolMeta
+from trytond.transaction import Transaction
+
+__all__ = ['InvoiceLine']
+__metaclass__ = PoolMeta
 
 
-class InvoiceLine(ModelSQL, ModelView):
-    _name = 'account.invoice.line'
+class InvoiceLine:
+    __name__ = 'account.invoice.line'
 
-    def _get_anglo_saxon_move_lines(self, line, amount, type_):
+    def _get_anglo_saxon_move_lines(self, amount, type_):
         '''
         Return account move for anglo-saxon stock accounting
         '''
@@ -18,7 +21,7 @@ class InvoiceLine(ModelSQL, ModelView):
 
         result = []
         move_line = {}
-        move_line['name'] = line.description
+        move_line['description'] = self.description
         move_line['amount_second_currency'] = Decimal('0.0')
         move_line['second_currency'] = None
 
@@ -30,7 +33,7 @@ class InvoiceLine(ModelSQL, ModelView):
             move_line['debit'] = Decimal('0.0')
             move_line['credit'] = amount
             account_type = type_[4:]
-        move_line['account'] = getattr(line.product,
+        move_line['account'] = getattr(self.product,
                 'account_stock_%s_used' % account_type).id
 
         result.append(move_line)
@@ -38,53 +41,50 @@ class InvoiceLine(ModelSQL, ModelView):
         move_line['debit'], move_line['credit'] = \
                 move_line['credit'], move_line['debit']
         if type_.endswith('supplier'):
-            move_line['account'] = line.account.id
+            move_line['account'] = self.account.id
         else:
-            move_line['account'] = line.product.account_cogs_used.id
+            move_line['account'] = self.product.account_cogs_used.id
         result.append(move_line)
         return result
 
-    def get_move_line(self, line):
-        pool = Pool()
-        move_obj = pool.get('stock.move')
-        currency_obj = pool.get('currency.currency')
+    def get_move_line(self):
+        Move = Pool().get('stock.move')
 
-        result = super(InvoiceLine, self).get_move_line(line)
+        result = super(InvoiceLine, self).get_move_line()
 
-        if line.type != 'line':
+        if self.type != 'line':
             return result
-        if not line.product:
+        if not self.product:
             return result
-        if line.product.type != 'goods':
+        if self.product.type != 'goods':
             return result
 
         moves = []
         # other types will get current cost price
-        if line.invoice.type == 'in_invoice':
-            moves = [move for purchase_line in line.purchase_lines
+        if self.invoice.type == 'in_invoice':
+            moves = [move for purchase_line in self.purchase_lines
                     for move in purchase_line.moves
                     if move.state == 'done']
-        elif line.invoice.type == 'out_invoice':
-            moves = [move for sale_line in line.sale_lines
+        elif self.invoice.type == 'out_invoice':
+            with Transaction().set_user(0, set_context=True):
+                sale_lines = self.__class__(self.id).sale_lines
+            moves = [move for sale_line in sale_lines
                     for move in sale_line.moves
                     if move.state == 'done']
-        if line.invoice.type == 'in_invoice':
+        if self.invoice.type == 'in_invoice':
             type_ = 'in_supplier'
-        elif line.invoice.type == 'out_invoice':
+        elif self.invoice.type == 'out_invoice':
             type_ = 'out_customer'
-        elif line.invoice.type == 'in_credit_note':
+        elif self.invoice.type == 'in_credit_note':
             type_ = 'out_supplier'
-        elif line.invoice.type == 'out_credit_note':
+        elif self.invoice.type == 'out_credit_note':
             type_ = 'in_customer'
 
         moves.sort(key=operator.attrgetter('effective_date'))
-        cost = move_obj.update_anglo_saxon_quantity_product_cost(
-                line.product, moves, line.quantity, line.unit, type_)
-        cost = currency_obj.round(line.invoice.currency, cost)
+        cost = Move.update_anglo_saxon_quantity_product_cost(
+            self.product, moves, self.quantity, self.unit, type_)
+        cost = self.invoice.currency.round(cost)
 
-        anglo_saxon_move_lines = self._get_anglo_saxon_move_lines(line, cost,
-                type_)
+        anglo_saxon_move_lines = self._get_anglo_saxon_move_lines(cost, type_)
         result.extend(anglo_saxon_move_lines)
         return result
-
-InvoiceLine()
