@@ -1,12 +1,15 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-from trytond.model import ModelView, ModelSQL, fields
+from trytond.model import fields
 from trytond.pyson import Eval
+from trytond.pool import PoolMeta
+
+__all__ = ['Category', 'Template']
+__metaclass__ = PoolMeta
 
 
-class Category(ModelSQL, ModelView):
-    _name = 'product.category'
-
+class Category:
+    __name__ = 'product.category'
     account_cogs = fields.Property(fields.Many2One('account.account',
             'Account Cost of Goods Sold', domain=[
                 ('kind', '!=', 'view'),
@@ -20,12 +23,9 @@ class Category(ModelSQL, ModelView):
     account_cogs_used = fields.Function(fields.Many2One('account.account',
         'Account Cost of Goods Sold Used'), 'get_account')
 
-Category()
-
-
-class Template(ModelSQL, ModelView):
-    _name = 'product.template'
 
+class Template:
+    __name__ = 'product.template'
     account_cogs = fields.Property(fields.Many2One('account.account',
             'Account Cost of Goods Sold', domain=[
                 ('kind', '!=', 'view'),
@@ -33,7 +33,8 @@ class Template(ModelSQL, ModelView):
                 ],
             states={
                 'invisible': ((~Eval('context', {}).get('company'))
-                    | Eval('account_category')),
+                    | Eval('account_category')
+                    | (Eval('type') != 'goods')),
                 'required': ((Eval('type') == 'goods')
                     & Eval('context', {}).get('company')
                     & ~Eval('account_category')),
@@ -43,5 +44,3 @@ class Template(ModelSQL, ModelView):
             depends=['account_category', 'type']))
     account_cogs_used = fields.Function(fields.Many2One('account.account',
         'Account Cost of Goods Sold Used'), 'get_account')
-
-Template()
